/*
 * linux/arch/arm/mm/discontig.c
 *
 * Discontiguous memory support.
 *
 * Initial code: Copyright (C) 1999-2000 Nicolas Pitre
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#include <linux/module.h>
#include <linux/mmzone.h>
#include <linux/bootmem.h>

#if MAX_NUMNODES != 4 && MAX_NUMNODES != 16 && MAX_NUMNODES != 64 && MAX_NUMNODES != 32
# error Fix Me Please
#endif

/*
 * Our node_data structure for discontiguous memory.
 */

static bootmem_data_t node_bootmem_data[MAX_NUMNODES];

pg_data_t discontig_node_data[MAX_NUMNODES] = {
  { .bdata = &node_bootmem_data[0] },
  { .bdata = &node_bootmem_data[1] },
  { .bdata = &node_bootmem_data[2] },
  { .bdata = &node_bootmem_data[3] },
#if MAX_NUMNODES == 16
  { .bdata = &node_bootmem_data[4] },
  { .bdata = &node_bootmem_data[5] },
  { .bdata = &node_bootmem_data[6] },
  { .bdata = &node_bootmem_data[7] },
  { .bdata = &node_bootmem_data[8] },
  { .bdata = &node_bootmem_data[9] },
  { .bdata = &node_bootmem_data[10] },
  { .bdata = &node_bootmem_data[11] },
  { .bdata = &node_bootmem_data[12] },
  { .bdata = &node_bootmem_data[13] },
  { .bdata = &node_bootmem_data[14] },
  { .bdata = &node_bootmem_data[15] },
#endif

#if MAX_NUMNODES == 32
  { .bdata = &node_bootmem_data[4] },
  { .bdata = &node_bootmem_data[5] },
  { .bdata = &node_bootmem_data[6] },
  { .bdata = &node_bootmem_data[7] },
  { .bdata = &node_bootmem_data[8] },
  { .bdata = &node_bootmem_data[9] },
  { .bdata = &node_bootmem_data[10] },
  { .bdata = &node_bootmem_data[11] },
  { .bdata = &node_bootmem_data[12] },
  { .bdata = &node_bootmem_data[13] },
  { .bdata = &node_bootmem_data[14] },
  { .bdata = &node_bootmem_data[15] },

  { .bdata = &node_bootmem_data[16] },
  { .bdata = &node_bootmem_data[17] },
  { .bdata = &node_bootmem_data[18] },
  { .bdata = &node_bootmem_data[19] },
  { .bdata = &node_bootmem_data[20] },
  { .bdata = &node_bootmem_data[21] },
  { .bdata = &node_bootmem_data[22] },
  { .bdata = &node_bootmem_data[23] },
  { .bdata = &node_bootmem_data[24] },
  { .bdata = &node_bootmem_data[25] },
  { .bdata = &node_bootmem_data[26] },
  { .bdata = &node_bootmem_data[27] },
  { .bdata = &node_bootmem_data[28] },
  { .bdata = &node_bootmem_data[29] },
  { .bdata = &node_bootmem_data[30] },
  { .bdata = &node_bootmem_data[31] },

#endif


#if MAX_NUMNODES == 64
  { .bdata = &node_bootmem_data[4] },
  { .bdata = &node_bootmem_data[5] },
  { .bdata = &node_bootmem_data[6] },
  { .bdata = &node_bootmem_data[7] },
  { .bdata = &node_bootmem_data[8] },
  { .bdata = &node_bootmem_data[9] },
  { .bdata = &node_bootmem_data[10] },
  { .bdata = &node_bootmem_data[11] },
  { .bdata = &node_bootmem_data[12] },
  { .bdata = &node_bootmem_data[13] },
  { .bdata = &node_bootmem_data[14] },
  { .bdata = &node_bootmem_data[15] },

  { .bdata = &node_bootmem_data[16] },
  { .bdata = &node_bootmem_data[17] },
  { .bdata = &node_bootmem_data[18] },
  { .bdata = &node_bootmem_data[19] },
  { .bdata = &node_bootmem_data[20] },
  { .bdata = &node_bootmem_data[21] },
  { .bdata = &node_bootmem_data[22] },
  { .bdata = &node_bootmem_data[23] },
  { .bdata = &node_bootmem_data[24] },
  { .bdata = &node_bootmem_data[25] },
  { .bdata = &node_bootmem_data[26] },
  { .bdata = &node_bootmem_data[27] },
  { .bdata = &node_bootmem_data[28] },
  { .bdata = &node_bootmem_data[29] },
  { .bdata = &node_bootmem_data[30] },
  { .bdata = &node_bootmem_data[31] },

  { .bdata = &node_bootmem_data[32] },
  { .bdata = &node_bootmem_data[33] },
  { .bdata = &node_bootmem_data[34] },
  { .bdata = &node_bootmem_data[35] },
  { .bdata = &node_bootmem_data[36] },
  { .bdata = &node_bootmem_data[37] },
  { .bdata = &node_bootmem_data[38] },
  { .bdata = &node_bootmem_data[39] },
  { .bdata = &node_bootmem_data[40] },
  { .bdata = &node_bootmem_data[41] },
  { .bdata = &node_bootmem_data[42] },
  { .bdata = &node_bootmem_data[43] },
  { .bdata = &node_bootmem_data[44] },
  { .bdata = &node_bootmem_data[45] },
  { .bdata = &node_bootmem_data[46] },
  { .bdata = &node_bootmem_data[47] },

  { .bdata = &node_bootmem_data[48] },
  { .bdata = &node_bootmem_data[49] },
  { .bdata = &node_bootmem_data[50] },
  { .bdata = &node_bootmem_data[51] },
  { .bdata = &node_bootmem_data[52] },
  { .bdata = &node_bootmem_data[53] },
  { .bdata = &node_bootmem_data[54] },
  { .bdata = &node_bootmem_data[55] },
  { .bdata = &node_bootmem_data[56] },
  { .bdata = &node_bootmem_data[57] },
  { .bdata = &node_bootmem_data[58] },
  { .bdata = &node_bootmem_data[59] },
  { .bdata = &node_bootmem_data[60] },
  { .bdata = &node_bootmem_data[61] },
  { .bdata = &node_bootmem_data[62] },
  { .bdata = &node_bootmem_data[63] },

#endif
};

EXPORT_SYMBOL(discontig_node_data);
