/*
 *  Driver routines for BUFFALO Platform
 *
 *  Copyright (C)  BUFFALO INC.
 *
 *  This software may be used and distributed according to the terms of
 *  the GNU General Public License (GPL), incorporated herein by reference.
 *  Drivers based on or derived from this code fall under the GPL and must
 *  retain the authorship, copyright and license notice.  This file is not
 *  a complete program and may only be used when the entire operating
 *  system is licensed under the GPL.
 *
 */


#include <linux/module.h>
#include <linux/proc_fs.h>
#include <linux/init.h>
#include <asm/uaccess.h>

/* Globals */
// same as Buffalo Kernel Ver.
#define TW_DRIVER_VERSION "0.08"

/* Module parameters */
MODULE_AUTHOR("BUFFALO");
MODULE_DESCRIPTION("Buffalo Platform Linux Driver");
MODULE_LICENSE("GPL");
MODULE_VERSION(TW_DRIVER_VERSION);

/* Definitions */
//#define DEBUG

#define USE_PROC_BUFFALO

// ----------------------------------------------------

/* Definitions for DEBUG */
#ifdef DEBUG
 #define FUNCTRACE(x)  x

#else
 #define FUNCTRACE(x) 

#endif


/* Function prototypes */


/* variables */
static struct proc_dir_entry *proc_buffalo;

#ifdef DEBUG
//----------------------------------------------------------------------
static int debug_read_proc(char *page, char **start, off_t offset, int length)
{
	//int              i;
	int              len  = 0;
	char*            buf  = page;
	off_t begin = 0;
	volatile struct tagReg {
		unsigned MainIntCause;
		unsigned MainIrqIntMask;
		unsigned MainFiqIntMask;
	} *intreg;
	unsigned base=0xF1000000;
	volatile unsigned *gpio=(unsigned *)(base+0x10100);
	
	intreg = (struct tagReg *)(base + 0x20200);
	
	len += sprintf(buf+len,"cause=%lx irqmask=%lx fiqmask=%lx\n "
		,intreg->MainIntCause, intreg->MainIrqIntMask,intreg->MainFiqIntMask );
	
	len += sprintf(buf+len,"GPIO:DO=%x DOutEn=%x DBlink=%x DatainActLow=%x DIn=%x Int=%x Mask=%x LMask=%x\n "
		,gpio[0],gpio[1],gpio[2],gpio[3],gpio[4],gpio[5],gpio[6],gpio[7] );
	
	*start = page + (offset - begin);	/* Start of wanted data */
	len -= (offset - begin);	/* Start slop */
	if (len > length)
		len = length;	/* Ending slop */
	return (len);
}

#endif

/* Functions */
//----------------------------------------------------------------------
char * getdate(void)
{
	static char newstr[32];
	char sMonth[8];
	unsigned char i;
	int iDay,iYear,iMonth=0;
	const char *months[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug",
                            "Sep", "Oct", "Nov", "Dec"};

	
	sscanf(__DATE__,"%s %d %d",sMonth,&iDay,&iYear);
	
	for (i = 0; i < 12; i++){
		if (!strcmp(sMonth, months[i])){
			iMonth = i + 1;
			break;
		}
	}
	sprintf(newstr,"%d/%02d/%02d",iYear,iMonth,iDay);
	return newstr;
}

//----------------------------------------------------------------------
static int kernelfw_read_proc(char *page, char **start, off_t offset, int length)
{
	//int              i;
	int              len  = 0;
	char*            buf  = page;
	off_t begin = 0;
	char *p,bootver[16];
	
	//printk("cmd:%s\n",saved_command_line);
	p=strstr(saved_command_line,"BOOTVER=");
	if (p){
		strncpy(bootver,p,sizeof(bootver));
	}
	len += sprintf(buf+len,"PRODUCTNAME=LS-GL(IESADA)\n");
	len += sprintf(buf+len,"VERSION=%s\n",TW_DRIVER_VERSION);
	len += sprintf(buf+len,"SUBVERSION=FLASH 0.00\n");
	len += sprintf(buf+len,"PRODUCTID=0x00000009\n");
	
	len += sprintf(buf+len,"BUILDDATE=%s %s\n",getdate(),__TIME__);
	len += sprintf(buf+len,"%s\n",bootver);
	
	*start = page + (offset - begin);	/* Start of wanted data */
	len -= (offset - begin);	/* Start slop */
	if (len > length)
		len = length;	/* Ending slop */
	return (len);
}
//----------------------------------------------------------------------
static int enetinfo_read_proc(char *page, char **start, off_t offset, int length)
{
	//int              i;
	int              len  = 0;
	char*            buf  = page;
	off_t begin = 0;
	int egiga_buffalo_proc_string(char *buf); // arch/arm/mach-mv88fxx81/egiga/mv_e_main.c

	len = egiga_buffalo_proc_string(buf);
	*start = page + (offset - begin);	/* Start of wanted data */
	len -= (offset - begin);	/* Start slop */
	if (len > length)
		len = length;	/* Ending slop */
	return (len);
}

//----------------------------------------------------------------------
int __init buffaloDriver_init (void)
{
	struct proc_dir_entry *generic;
	FUNCTRACE(printk(">%s\n",__FUNCTION__));
	
	proc_buffalo = proc_mkdir("buffalo", 0);
	if (!proc_buffalo) {
		printk (KERN_ERR "cannot init /proc/buffalo\n");
		return -ENOMEM;
	}
	generic = create_proc_info_entry ("buffalo/firmware", 0, 0, kernelfw_read_proc);
#ifdef DEBUG
	generic = create_proc_info_entry ("buffalo/debug", 0, 0, debug_read_proc);
#endif
	generic = create_proc_info_entry ("buffalo/enet", 0, 0, enetinfo_read_proc);
	return 0;
}

//----------------------------------------------------------------------
void buffaloDriver_exit(void)
{
	FUNCTRACE(printk(">%s\n",__FUNCTION__));
	
	remove_proc_entry("buffalo/enet", 0);
#ifdef DEBUG
	remove_proc_entry("buffalo/debug", 0);
#endif
	remove_proc_entry("buffalo/firmware", 0);
	remove_proc_entry ("buffalo", 0);
}

module_init(buffaloDriver_init);
module_exit(buffaloDriver_exit);
