/* linux/drivers/ide/arm/ide-tcf.c
 *
 * Copyright (c) 2007 Daniel Smolik
 *  Mydatex  <marvin@mydatex.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
*/

#include <linux/module.h>
#include <linux/errno.h>
#include <linux/ide.h>
#include <linux/init.h>

#include <asm/mach-types.h>

#include <asm/io.h>
#include <asm/irq.h>

//#include <asm/arch/ts72xx.h>

/*      TS7200 CF memory map:                                                        
*                                                                              
* phys         size    description                                             
* 11000001     7       CF registers (8-bit)                                    
* 10400006     2       CF aux registers (8-bit)                                
* 21000000     2       CF data register (16-bit)                               
*/                                                                             
                                                                                      
#define TS7200_CF_CMD_PHYS_BASE         0x11000001                              
#define TS7200_CF_AUX_PHYS_BASE         0x10400006                              
#define TS7200_CF_DATA_PHYS_BASE        0x21000000





static ide_hwif_t *tscf_find_hwif(unsigned long dataport)
{
	ide_hwif_t *hwif;
	int index;

	for (index = 0; index < MAX_HWIFS; ++index) {
		hwif = &ide_hwifs[index];
		if (hwif->io_ports[IDE_DATA_OFFSET] == dataport)
			goto found;
	}

	for (index = 0; index < MAX_HWIFS; ++index) {
		hwif = &ide_hwifs[index];
		if (!hwif->io_ports[IDE_DATA_OFFSET])
			goto found;
	}

	hwif = NULL;
found:
	return hwif;
}







static int __init
tcf_register(unsigned long dataport,int irq)
{
	ide_hwif_t *hwif;
	//int i,ret;

	

	hwif = tscf_find_hwif(dataport);
	
	if (hwif) {
		//int i;

		memset(&hwif->hw, 0, sizeof(hw_regs_t));

		/*
		 * Ensure we're using MMIO
		 */
		default_hwif_mmiops(hwif);
		hwif->mmio = 1;

		hwif->io_ports[IDE_DATA_OFFSET] =    (unsigned long)ioremap(TS7200_CF_DATA_PHYS_BASE,1);

		hwif->io_ports[IDE_ERROR_OFFSET] =   (unsigned long)ioremap(TS7200_CF_CMD_PHYS_BASE,1);
		hwif->io_ports[IDE_NSECTOR_OFFSET] = (unsigned long)ioremap(TS7200_CF_CMD_PHYS_BASE + 0x1,1);
		hwif->io_ports[IDE_SECTOR_OFFSET] =  (unsigned long)ioremap(TS7200_CF_CMD_PHYS_BASE + 0x2,1);
		hwif->io_ports[IDE_LCYL_OFFSET] =    (unsigned long)ioremap(TS7200_CF_CMD_PHYS_BASE + 0x3,1);
		hwif->io_ports[IDE_HCYL_OFFSET] =    (unsigned long)ioremap(TS7200_CF_CMD_PHYS_BASE + 0x4,1);
		hwif->io_ports[IDE_SELECT_OFFSET] =  (unsigned long)ioremap(TS7200_CF_CMD_PHYS_BASE + 0x5,1);
		hwif->io_ports[IDE_STATUS_OFFSET] =  (unsigned long)ioremap(TS7200_CF_CMD_PHYS_BASE + 0x6,1);
		
		hwif->io_ports[IDE_CONTROL_OFFSET] = (unsigned long)ioremap(TS7200_CF_AUX_PHYS_BASE,1);



		

		/*
		* This is the dma channel number assigned to this IDE interface. Until
		* dma is enabled for this interface, we set it to NO_DMA.
		*/
	   	hwif->hw.dma = NO_DMA;
	   	hwif->dma = NO_DMA;

 
		hwif->hw.irq  = irq;
		hwif->irq     = irq;
		hwif->noprobe = 0;
		hwif->chipset = ide_generic;
		//hwif->chipset = ide_forced;
		
		
		probe_hwif_init(hwif);                                                 
                                                                                                                      
                create_proc_ide_interfaces();
		//ide_proc_register_port(hwif);
	}

	 printk("Technologic Systems TS-7200 IDE initialization - driver version 1.5, 12/07/07. \n");

	 return 0;
}

static int __init tcf_init(void)
{

	if (! board_is_ts7200())
		return 0;


	tcf_register(TS7200_CF_DATA_PHYS_BASE,IRQ_EP93XX_EXT0);
	return 0;
}

module_init(tcf_init);

MODULE_AUTHOR("Dan Smolik <marvin@mydatex.cz>");
MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("TS-7200  IDE driver");

